module calculos

! objetivo: calcular as variveis

use dados

implicit none

contains

!-------------------------------------------------------------------------------

  subroutine calcula

    call extrapolacoes_e_ordens_aparente

    call extrapolacao_bicoeficiente

    call extrapolacao_tricoeficiente

    pE_h   = great
    pE_i   = great
    pE_c12 = great
    pE_c13 = great
    pE_bi  = great
    if ( com_solucao_analitica == 1 ) then
       call erros_verdadeiros 
       call ordens_efetiva
    end if

    call estimador_delta

    call estimador_Richardson

    call estimador_GCI

    call multiextrapolacao

  end subroutine calcula

!-------------------------------------------------------------------------------

  subroutine extrapolacoes_e_ordens_aparente
  
    real*16 :: psi ! taxa de convergncia
    real*16 :: del ! auxiliar

    ! pU_h = ordem aparente com base em Th
    pU_h = great
 	it_pU_h = 0
    if ( nm >= 3 ) then
       do i = 3, nm
          del = Th(i) - Th(i-1)
          if ( del /= 0.0q0 ) then
             psi = (Th(i-1)-Th(i-2))/(Th(i)-Th(i-1))
          else
             psi = 0.0q0
          end if
          if ( tipo_refino == 1 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pU_h(i) = qlog10(psi)/qlog10(r21)
          end if
          if ( tipo_refino == 2 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             r32 = h(i-2) / h(i-1)
             call bissecao ( pU_min, pU_max, r21, r32, psi, it_pU_h(i), pU_h(i) )
          end if
       end do
    end if

    ! Ti_12 = Th extrapolado com as malhas 1 e 2 e pL
    do i = 2, nm
       r21 = h(i-1) / h(i)
       Ti_12(i) = Th(i) + (Th(i) - Th(i-1)) / ((r21**pL) - 1.0q0)
    end do

    ! Ti_13 = Th extrapolado com as malhas 1 e 3 e pL
    ! Ti_23 = Th extrapolado com as malhas 2 e 3 e pL
    ! Ti_pU = Th extrapolado com as malhas 1 a 3 e pU_h
    ! Tc_12 = Mdia de Ti_12 com Ti_pU
    ! Tc_13 = Mdia de Ti_13 com Ti_pU
    if ( nm >= 3 ) then
       do i = 3, nm
          r21 = h(i-1) / h(i)
          r32 = h(i-2) / h(i-1)
          Ti_23(i) = Th(i-1) + (Th(i-1) - Th(i-2)) / ((r32**pL) - 1.0q0)
          Ti_13(i) = Th(i)   + (Th(i)   - Th(i-2)) / (((r21*r32)**pL) - 1.0q0)
          if ( pU_h(i) <= 0.0q0 .or. pU_h(i) > limite ) then
             Ti_pU(i) = great
             Tc_12(i) = great
             Tc_13(i) = great
          else
             Ti_pU(i) = Th(i) + (Th(i) - Th(i-1)) / ((r21**pU_h(i)) - 1.0q0)
             Tc_12(i) = ( Ti_12(i) + Ti_pU(i) ) / 2
             Tc_13(i) = ( Ti_13(i) + Ti_pU(i) ) / 2
          end if
       end do
    end if

    ! pU_i = ordem aparente com base em Ti_pU
    pU_i = great
    it_pU_i = 0
    if ( nm >= 5 ) then
       do i = 5, nm
          del = Ti_pU(i) - Ti_pU(i-1)
          if ( del /= 0.0q0 ) then
             psi = (Ti_pU(i-1)-Ti_pU(i-2))/(Ti_pU(i)-Ti_pU(i-1))
          else
             psi = 0.0q0
          end if
          if ( tipo_refino == 1 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pU_i(i) = qlog10(psi)/qlog10(r21)
          end if
          if ( tipo_refino == 2 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             r32 = h(i-2) / h(i-1)
             call bissecao ( pU_min, pU_max, r21, r32, psi, it_pU_i(i), pU_i(i) )
          end if
       end do
    end if

    ! pU_c12 = ordem aparente com base em Tc_12
    pU_c12 = great
    it_pU_c12 = 0
    if ( nm >= 5 ) then
       do i = 5, nm
          del = Tc_12(i) - Tc_12(i-1)
          if ( del /= 0.0q0 ) then
             psi = (Tc_12(i-1)-Tc_12(i-2))/(Tc_12(i)-Tc_12(i-1))
          else
             psi = 0.0q0
          end if
          if ( tipo_refino == 1 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pU_c12(i) = qlog10(psi)/qlog10(r21)
          end if
          if ( tipo_refino == 2 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             r32 = h(i-2) / h(i-1)
             call bissecao ( pU_min, pU_max, r21, r32, psi, it_pU_c12(i), pU_c12(i) )
          end if
       end do
    end if

    ! pU_c13 = ordem aparente com base em Tc_13
    pU_c13 = great
    it_pU_c13 = 0
    if ( nm >= 5 ) then
       do i = 5, nm
          del = Tc_13(i) - Tc_13(i-1)
          if ( del /= 0.0q0 ) then
             psi = (Tc_13(i-1)-Tc_13(i-2))/(Tc_13(i)-Tc_13(i-1))
          else
             psi = 0.0q0
          end if
          if ( tipo_refino == 1 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pU_c13(i) = qlog10(psi)/qlog10(r21)
          end if
          if ( tipo_refino == 2 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             r32 = h(i-2) / h(i-1)
             call bissecao ( pU_min, pU_max, r21, r32, psi, it_pU_c13(i), pU_c13(i) )
          end if
       end do
    end if

    ! Tbi_pU = Th biextrapolado com malhas 1 a 5 e pU_i
    if ( nm >= 5 ) then
       do i = 5, nm
          r21 = h(i-1) / h(i)
          if ( pU_i(i) <= 0.0q0 .or. pU_i(i) > limite ) then
             Tbi_pU(i) = great
          else
             Tbi_pU(i) = Ti_pU(i) + (Ti_pU(i) - Ti_pU(i-1)) / ((r21**pU_i(i)) - 1.0q0)
          end if
       end do
    end if

    ! pU_bi = ordem aparente com base em Tbi_pU
    pU_bi    = great
    it_pU_bi = 0
    if ( nm >= 7 ) then
       do i = 7, nm
          del = Tbi_pU(i) - Tbi_pU(i-1)
          if ( del /= 0.0q0 ) then
             psi = (Tbi_pU(i-1)-Tbi_pU(i-2))/(Tbi_pU(i)-Tbi_pU(i-1))
          else
             psi = 0.0q0
          end if
          if ( tipo_refino == 1 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pU_bi(i) = qlog10(psi)/qlog10(r21)
          end if
          if ( tipo_refino == 2 .and. psi > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             r32 = h(i-2) / h(i-1)
             call bissecao ( pU_min, pU_max, r21, r32, psi, it_pU_bi(i), pU_bi(i) )
          end if
       end do
    end if

  end subroutine extrapolacoes_e_ordens_aparente

!-------------------------------------------------------------------------------

  subroutine bissecao ( pUi, pUs, r21, r32, psi, i_pU, pUm )

    integer :: it   ! nmero de iteraes corrente na obteno de pU

    real*16 :: f_pUi ! resto da funo de pU para pUi
    real*16 :: f_pUm ! resto da funo de pU para pUm
    real*16 :: dp    ! intervalo de teste de pU
    real*16 :: pi    ! limite inferior corrente de pU
    real*16 :: ps    ! limite superior corrente de pU

    real*16,intent(in) :: pUi ! limite inferior de pU (dado inicial)
    real*16,intent(in) :: pUs ! limite superior de pU (dado inicial)
    real*16,intent(in) :: r21 ! razo de h entre as malhas 2 e 1
    real*16,intent(in) :: r32 ! razo de h entre as malhas 3 e 2
    real*16,intent(in) :: psi ! razo de convergncia

    integer,intent(out) :: i_pU ! nmero de iteraes usado na obteno de pU
    real*16, intent(out) :: pUm  ! soluo de pU

    pi = pUi
    ps = pUs

    do it = 1, numero_iteracoes
      pUm = 0.5q0 * (pi + ps)
      f_pUi = pi  * qlog10(r21) - qlog10(psi*((r21**pi )-1.0q0)/((r32**pi )-1.0q0))
      f_pUm = pUm * qlog10(r21) - qlog10(psi*((r21**pUm)-1.0q0)/((r32**pUm)-1.0q0))
      if ( f_pUi * f_pUm > 0.0q0 ) then
         pi = pUm
      else
         dp = ps - pi
         i_pU = it
         if ( dp <= tol ) exit
         ps = pUm
      end if
    end do

    if ( it == numero_iteracoes .or. it == (numero_iteracoes+1) ) pUm = great

  end subroutine bissecao

!-------------------------------------------------

  subroutine extrapolacao_bicoeficiente

    real*16 :: aux1, aux2

    r21 = h(1) / h(2)

    aux1 = r21 ** (2*pL + dpL)

    aux2 = - (r21**pL) * ( r21**dpL + 1.0q0 )

    if ( nm >= 3 ) then

       do i = 3, nm

          Ti_bi(i) = ( aux1 * Th(i) + aux2 * Th(i-1) + Th(i-2) ) / &
                     ( aux1 + aux2 + 1.0q0 )

       end do

    end if

  end subroutine extrapolacao_bicoeficiente 

!-------------------------------------------------

  subroutine extrapolacao_tricoeficiente

    real*16 :: f1, f2, a1, a2, a3, a4, ai, q

    r21 = h(1) / h(2)

    q = r21

    f1 = (q**(3*dpL)) - (q**(2*dpL)) - (q**dpL) + 1.0q0

    f2 = ( (q**(6*dpL)) - (q**(5*dpL)) - (q**(4*dpL))     &
       +   (q**(2*dpL)) + (q**dpL)     - 1.0q0     ) * (q**pL)     

    a1 = (q**(2*pL+dpL)) * ( f2 + (q**pL)*(1.0q0-(q**(2*dpL)))*f1 )

    a2 = (q**pL) * ( (q**pL)*((q**(3*dpL))-1.0q0)*f1 - ((q**dpL)+1.0q0)*f2 )

    a3 = f2

    a4 = ( 1.0q0 - (q**dpL) ) * f1

    ai = f2 * ( (q**(2*pL+dpL)) + 1.0q0 - (q**(pL+dpL)) - (q**pL) ) &
       + f1 * ( (q**(2*pL+3*dpL)) + (q**(3*pL+dpL)) + 1.0q0         &
       -        (q**(3*pL+3*dpL)) - (q**(2*pL))     - (q**dpL)      )

    if ( nm >= 4 ) then

       do i = 4, nm

          Ti_tri(i) = ( a1 * Th(i) + a2 * Th(i-1) + a3 * Th(i-2) &
                    +   a4 * Th(i-3) ) / ai
       end do

    end if

  end subroutine extrapolacao_tricoeficiente 

!-------------------------------------------------------------------------------

  subroutine erros_verdadeiros
  
    ! Eh = erro verdadeiro de Th
    do i = 1, nm
       Eh(i) = T_exato - Th(i)
    end do

    ! Ei_12 = erro verdadeiro de Ti_12
    do i = 2, nm
       Ei_12(i) = T_exato - Ti_12(i)
    end do

    ! Ei_13 = erro verdadeiro de Ti_13
    ! Ei_23 = erro verdadeiro de Ti_23
    ! Ei_pU = erro verdadeiro de Ti_pU
    ! Ec_12 = erro verdadeiro de Tc_12
    ! Ec_13 = erro verdadeiro de Tc_13
    ! Ei_bi = erro verdadeiro de Ti_bi
    if ( nm >= 3 ) then
       do i = 3, nm
          Ei_23(i) = T_exato - Ti_23(i)
          Ei_13(i) = T_exato - Ti_13(i)
          Ei_bi(i) = T_exato - Ti_bi(i)
          if ( Ti_pU(i) > limite ) then
             Ei_pU(i) = great
             Ec_12(i) = great
             Ec_13(i) = great
          else
             Ei_pU(i) = T_exato - Ti_pU(i)
             Ec_12(i) = T_exato - Tc_12(i)
             Ec_13(i) = T_exato - Tc_13(i)
          end if
       end do
    end if

    ! Ei_tri = erro verdadeiro de Ti_tri
    if ( nm >= 4 ) then
       do i = 4, nm
          Ei_tri(i) = T_exato - Ti_tri(i)
       end do
    end if

    ! Ebi_pU = erro verdadeiro de Tbi_pU
    if ( nm >= 5 ) then
       do i = 5, nm
          if ( Tbi_pU(i) > limite ) then
             Ebi_pU(i) = great
          else
             Ebi_pU(i) = T_exato - Tbi_pU(i)
          end if
       end do
    end if

  end subroutine erros_verdadeiros

!-------------------------------------------------------------------------------

  subroutine ordens_efetiva

    ! pE_h = ordem efetiva do erro verdadeiro de Th
    do i = 2, nm
       if ( Eh(i) /= 0.0q0 ) then
          if ( Eh(i-1)/Eh(i) > 0.0q0 ) then
             r21 = h(i-1) / h(i)
             pE_h(i) = qlog10(Eh(i-1)/Eh(i))/qlog10(r21)
          end if
       end if
    end do

    ! pE_i = ordem efetiva do erro verdadeiro de Ti_pU
    if ( nm >= 4 ) then
       do i = 4, nm
          if ( Ei_pU(i) /= 0.0q0 ) then
             if ( Ei_pU(i-1)/Ei_pU(i) > 0.0q0 ) then
                r21 = h(i-1) / h(i)
                if ( Ei_pU(i) < limite ) then
                   pE_i(i) = qlog10(Ei_pU(i-1)/Ei_pU(i))/qlog10(r21)
                end if
             end if
          end if
       end do
    end if

    ! pE_c12 = ordem efetiva do erro verdadeiro de Tc_12
    if ( nm >= 4 ) then
       do i = 4, nm
          if ( Ec_12(i) /= 0.0q0 ) then
             if ( Ec_12(i-1)/Ec_12(i) > 0.0q0 ) then
                r21 = h(i-1) / h(i)
                if ( Ec_12(i) < limite ) then
                   pE_c12(i) = qlog10(Ec_12(i-1)/Ec_12(i))/qlog10(r21)
                end if
             end if
          end if
       end do
    end if

    ! pE_c13 = ordem efetiva do erro verdadeiro de Tc_13
    if ( nm >= 4 ) then
       do i = 4, nm
          if ( Ec_13(i) /= 0.0q0 ) then
             if ( Ec_13(i-1)/Ec_13(i) > 0.0q0 ) then
                r21 = h(i-1) / h(i)
                if ( Ec_13(i) < limite ) then
                   pE_c13(i) = qlog10(Ec_13(i-1)/Ec_13(i))/qlog10(r21)
                end if
             end if
          end if
       end do
    end if

    ! pE_bi = ordem efetiva do erro verdadeiro de Tbi_pU
    if ( nm >= 6 ) then
       do i = 6, nm
          if ( Ebi_pU(i) /= 0.0q0 ) then
             if ( Ebi_pU(i-1)/Ebi_pU(i) > 0.0q0 ) then
                r21 = h(i-1) / h(i)
                if ( Ebi_pU(i) < limite ) then
                   pE_bi(i) = qlog10(Ebi_pU(i-1)/Ebi_pU(i))/qlog10(r21)
                end if
             end if
          end if
       end do
    end if

  end subroutine ordens_efetiva

!-------------------------------------------------------------------------------

  subroutine estimador_delta

    ! Ud = erro estimado de Th
    do i = 2, nm
       Ud(i) = qabs(Th(i) - Th(i-1))
    end do

  end subroutine estimador_delta

!-------------------------------------------------------------------------------

  subroutine estimador_Richardson 

    ! Uri_12 = erro estimado de Th com as malhas 1 e 2 e pL
    do i = 2, nm
       Uri_12(i) = Ti_12(i) - Th(i)
    end do

    ! Uri_13 = erro estimado de Th com as malhas 1 e 3 e pL
    ! Uri_23 = erro estimado de Th com as malhas 2 e 3 e pL
    ! Uri_pU = erro estimado de Th com as malhas 1 a 3 e pU_h
    ! Uri_p  = erro estimado de Th com as malhas 1 a 3, pU_h e pL
    ! Uc_12  = erro estimado de Tc_12
    ! Uc_13  = erro estimado de Tc_13
    ! Uri_bi = erro estimado de Th extrapolado com as malhas 1 a 3 e 2 coeficientes
    if ( nm >= 3 ) then
       do i = 3, nm
          Uri_13(i) = Ti_13(i) - Th(i)
          Uri_23(i) = Ti_23(i) - Th(i)
          Uri_bi(i) = Ti_bi(i) - Th(i)
          if ( Ti_pU(i) > limite ) then
             Uri_pU(i) = great
             Uri_p(i)  = great
             Uc_12(i)  = great
             Uc_13(i)  = great
          else
             Uri_pU(i) = Ti_pU(i) - Th(i)
             Uri_p(i)  = qsign ( qmax1 ( qabs(Uri_12(i)), qabs(Uri_pU(i)) ), &
                                       ( Th(i) - Th(i-1) ) )
             Uc_12(i)  = qabs ( Ti_12(i) - Ti_pU(i) ) / 2
             Uc_13(i)  = qabs ( Ti_13(i) - Ti_pU(i) ) / 2
          end if
       end do
    end if

    ! Uri_tri = erro estimado de Th extrapolado com as malhas 1 a 4 e 3 coeficientes
    if ( nm >= 4 ) then
       do i = 4, nm
          Uri_tri(i) = Ti_tri(i) - Th(i)
       end do
    end if

  end subroutine estimador_Richardson

!-------------------------------------------------------------------------------

  subroutine estimador_GCI

    real*16 :: p ! auxiliar

    ! GCI_pL = erro estimado de Th com as malhas 1 e 2 e pL
    do i = 2, nm
       r21 = h(i-1) / h(i)
       GCI_pL(i) = Fs * qabs ( Th(i) - Th(i-1) ) / ( r21**pL - 1.0q0 )
    end do

    ! GCI_p  = erro estimado de Th com as malhas 1 a 3, pU_h e pL
    if ( nm >= 3 ) then
       do i = 3, nm
          if ( Ti_pU(i) > limite ) then
             GCI_p(i)  = great
          else
             r21 = h(i-1) / h(i)
             p = qmin1 ( pL, pU_h(i) )
             GCI_p(i) = Fs * qabs ( Th(i) - Th(i-1) ) / ( r21**p - 1.0q0 )
          end if
       end do
    end if

  end subroutine estimador_GCI

!-------------------------------------------------------------------------------

  subroutine multiextrapolacao

    ! Tm_12 = Th multiextrapolado com as malhas 1 e 2 e pV

    m = 1
    do i = 2, nm
       Tm_12(i,m) = Ti_12(i)
    end do

    do m = 2, nm-1

       do i = m+1, nm

          r21 = h(i-1) / h(i)

          Tm_12(i,m) = Tm_12(i,m-1) + ( Tm_12(i,m-1) - Tm_12(i-1,m-1) ) &
                     / (  r21 ** pV(m) - 1.0q0 )

       end do

    end do

    ! Um_12 = erro estimado de Tm_12 com as malhas 1 e 2 e pV

    do m = 1, nm-2

       do i = m+2, nm

          Um_12(i,m) = Tm_12(i,m+1) - Tm_12(i,m) 

       end do

    end do

  end subroutine multiextrapolacao

!-------------------------------------------------------------------------------

end module calculos