module dados

! objetivo: ler e escrever os dados do programa

use variaveis

implicit none

contains

!-------------------------------------------------------------------------------

  subroutine le_dados

    ! leitura do nome do arquivo principal de dados do CASO a analisar
    dos = system('notepad Richardson_3p0.in') ! lista arquivo
    open(in,file='Richardson_3p0.in')
    read(in,*) arq_dados
	close(in)

    ! leitura dos dados principais
    note_caso = 'notepad ' // trim(adjustl(arq_dados))
    dos = system( note_caso ) ! lista arquivo
    open(in,file=trim(adjustl(arq_dados)))

    read(in,*) variavel
    read(in,*) caso
    read(in,*) com_solucao_analitica
    read(in,*) arq_exato
    read(in,*) pL
    read(in,*) dpL
    read(in,*) Fs
    read(in,*) tipo_refino
    read(in,*) numero_iteracoes
    read(in,*) tol
    read(in,*) pU_min
    read(in,*) pU_max
    read(in,*) nm

    ! alocao de memria
    allocate ( h(nm), Th(nm), Eh(nm), pV(nm), pVt(nm) )
    allocate ( Ti_12(nm), Ti_13(nm), Ti_23(nm), Ti_pU(nm),  Tbi_pU(nm), &
               Tc_12(nm), Tc_13(nm), Ti_bi(nm), Ti_tri(nm)  )
    allocate ( Ei_12(nm), Ei_13(nm), Ei_23(nm), Ei_pU(nm),  Ebi_pU(nm), &
               Ec_12(nm), Ec_13(nm), Ei_bi(nm), Ei_tri(nm)  )
    allocate ( pE_h(nm),  pE_i(nm),  pE_bi(nm), pE_c12(nm), pE_c13(nm), &
               pU_h(nm),  pU_i(nm),  pU_bi(nm), pU_c12(nm), pU_c13(nm)  )
    allocate ( Ud(nm), Uri_12(nm), Uri_13(nm), Uri_23(nm), Uri_pU(nm),  &
                       Uc_12(nm),  Uc_13(nm),  Uri_bi(nm), Uri_tri(nm), &
                       Uri_p(nm),  GCI_p(nm),  GCI_pL(nm)  )
    allocate ( it_pU_h(nm), it_pU_i(nm), it_pU_bi(nm), it_pU_c12(nm), it_pU_c13(nm) )
    allocate ( arq_numerico(nm) )
    allocate ( Tm_12(nm,nm-1), Um_12(nm,nm-1) )

    ! leitura dos nomes dos arquivos com resultados numricos

    do i = 1, nm
       read(in,*) arq_numerico(i)
       pV(i) = pL + (i-1)*dpL
       write(pVt(i),1) pV(i)
       1 format(f5.2)
    end do

    read(in,*) title

	close(in)

    ! leitura da soluo analtica exata
    if ( com_solucao_analitica == 1 ) then
       open(in,file=trim(adjustl(arq_exato)))
       do k = 1, variavel
          read(in,*) j, T_exato
       end do
       close(in)
    end if

    ! leitura dos resultados numricos nas nm malhas
    do i = 1, nm
       open(in,file=trim(adjustl(arq_numerico(i))))
       read(in,*) h(i)
       do k = 1, variavel
          read(in,*) j, Th(i)
       end do
    end do
	close(in)

  end subroutine le_dados

!-------------------------------------------------------------------------------

  subroutine escreve_dados

    write(out,1) trim(adjustl(title)), dia, hora, variavel, arq_dados, pL, dpl, nm

    1 format(/, 'Listagem gerada pelo aplicativo RICHARDSON 3.1, de 12 Dez 2007', //, &
        3x, 'ttulo = ', a,//, &
        3x, 'Dia = ',a12, 5x, 'Hora = ',a8, //, &
        i4, ' = nmero da varivel, nos arquivos de resultados, a analisar', //, &
       'nome do arquivo de dados principal = ', a, //, &
        f5.2, ' = ordem assinttica do erro verdadeiro', /, &
        f5.2, ' = variao entre as ordens verdadeiras do erro verdadeiro', //, &
        i4, ' = nmero de malhas' )

    if ( com_solucao_analitica == 1 ) write(out,7) trim(adjustl(arq_exato)), T_exato
        7 format (/, 'nome do arquivo da soluo analtica = ', a, //,  &
           1pe40.32, ' = soluo analtica')

    if ( tipo_refino == 1 ) then 
       write(out,9)
       9 format (/, 'O refino de malha (r)  constante')
    else
       write(out,8) numero_iteracoes, tol, pU_min, pU_max
       8 format (/, 'O refino de malha (r)  varivel', /, &
          i4, ' = nmero mximo de iteraes para obter pU com r varivel', /, &
          1pe13.4, ' = tolerncia na obteno de pU com r varivel', /, &
          1pe13.4, ' = limite inferior de pU (r varivel)', /, &
          1pe13.4, ' = limite superior de pU (r varivel)')
    end if

       write(out,2)
       2 format (/, t3,'h', t32,'log10(h)', t45,'r21', t58,'arquivos com resultados numricos:' )

       i = 1
       write(out,3) h(i), qlog10(h(i)), trim(adjustl(arq_numerico(i)))

       do i = 2, nm
          r21 = h(i-1) / h(i)
          write(out,4) h(i), qlog10(h(i)), r21, trim(adjustl(arq_numerico(i)))
       end do

       3 format(1pe28.20,1pe13.4,16x,a)

       4 format(1pe28.20,2(1pe13.4),3x,a)

  end subroutine escreve_dados

!-------------------------------------------------------------------------------

end module dados