program Richardson_3p2

!  Analisador e estimador de erros de discretizao
!     com base na extrapolao de Richardson

!  Verso original 1.0 ( 1 Fev 04)
!  Verso          2.0 (10 Jan 07)
!  Verso          1.1 (30 Abr 07)
!  Verso          3.0 (10 Nov 07)
!  Verso          3.1 ( 2 Dez 07)
!  Verso atual    3.2 (10 Jul 09)
!  ltima alterao =   10 Jul 09

!  Autor: Carlos Henrique Marchi
!         DEMEC/TC/UFPR, Curitiba, PR

!  MODELO NUMRICO (resumo)

!     Faz extrapolaes simples e mltiplas dos resultados numricos
!     Calcula ordens efetiva e aparente de resultados extrapolados ou no
!     Calcula erro estimado de resultados numricos com e sem extrapolaes,
!        com estimadores delta, Richardson, GCI, convergente e multicoeficientes
!     Se a soluo exata est disponvel, calcula erros verdadeiros e o
!        desempenho dos estimadores
!     Razo de refino de malha (r) pode ser constante ou varivel;
!        mtodo da bisseo para obter ordens aparente com r varivel
!     Malhas uniformes
!     Preciso qudrupla
!     Linguagem Fortran 2003
!     Tipo de projeto: Console Application
!     Aplicativo usado: Fortran Intel 11.0
!     Extrapolaes bi e tricoeficiente s se aplicam para refino constante
!     Os dados devem conter no mnimo duas malhas
!     Os valores de pL e dpL so necessrios mesmo que a soluo analtica
!        no seja conhecida; seus valores podem ser reais
!     Resultados em branco de parmetros significa que no h dados suficientes
!        para calcul-lo ou no  possvel calcul-lo

! ARQUIVOS envolvidos no programa:

!    Fontes (prefixo Richardson_3p2_):
!       data_hora.f90  = obtm a data e hora do sistema
!       variaveis.f90  = define todas as variveis globais do programa
!       dados.f90      = l e escreve os dados do programa
!       calculos.f90   = calcula as variveis
!       resultados.f90 = escreve resultados e apresenta grficos
!       main.f90       = programa principal
      
!    Dados:
!       Richardson_3p0.in  = leitura do nome do arquivo principal de dados do CASO a analisar
!       "arq_dados"        = leitura dos dados principais
!       "arq_exato"        = nome do arquivo da soluo exata
!       "arq_numerico"     = nomes dos arquivos das solues numricas (um arquivo para cada
!                            malha mas com os resultados de todas as variveis de interesse )

!    Sada principal:
!       "caso"             = sada principal do programa

!    Aplicativos auxiliares:
!       Notepad.exe        = editor de arquivos
!       Wgnuplot.exe       = programa gerador de grfico

!    Arquivos de sada auxiliares para grficos:
!       pL.dat     = dados para grfico de pL
!       p2.dat     = dados para grfico de pV(2)
!       p3.dat     = dados para grfico de pV(3)
!       pE_h.dat   = dados para grfico de pE_h
!       pE_i.dat   = dados para grfico de pE_i
!       pE_bi.dat  = dados para grfico de pE_bi
!       pE_c12.dat = dados para grfico de pE_c12
!       pE_c13.dat = dados para grfico de pE_c13
!       pU_h.dat   = dados para grfico de pU_h
!       pU_i.dat   = dados para grfico de pU_i
!       pU_bi.dat  = dados para grfico de pU_bi
!       pU_c12.dat = dados para grfico de pU_c12
!       pU_c13.dat = dados para grfico de pU_c13
!       Eh.dat     = dados para grfico de Eh
!       Ei_12.dat  = dados para grfico de Ei_12
!       Ei_pU.dat  = dados para grfico de Ei_pU
!       Ec_12.dat  = dados para grfico de Ec_12
!       Ec_13.dat  = dados para grfico de Ec_13
!       Ei_13.dat  = dados para grfico de Ei_13
!       Ei_23.dat  = dados para grfico de Ei_23
!       Ei_bi.dat  = dados para grfico de Ei_bi
!       Ei_tri.dat = dados para grfico de Ei_tri
!       Ebi.dat    = dados para grfico de Ebi_pU
!       U_12.dat   = dados para grfico de Ud, Uri_12 e GCI_pL
!       Uri_13.dat = dados para grfico de Uri_13
!       Uri_23.dat = dados para grfico de Uri_23
!       Uri_bi.dat = dados para grfico de Uri_bi
!       Utri.dat   = dados para grfico de Uri_tri
!       Uri_pU.dat = dados para grfico de Uri_pU, Uri_p, GCI_p
!       Uc_12.dat  = dados para grfico de Uc_12
!       Uc_13.dat  = dados para grfico de Uc_13
!       E_pL_p2.dat= auxiliar para grficos de E
!       E_p3.dat   = auxiliar para grficos de E
!       U_pL.dat   = auxiliar para grficos de U
!       U_p2.dat   = auxiliar para grficos de U
!       U_p3.dat   = auxiliar para grficos de U

!    Arquivos de comandos para Wgnuplot (prefixo Richardson_3p0_):
!       pU.gnu    = grfico de pU
!       pE_pU.gnu = grfico de pE e pU
!       U.gnu     = grfico de erros estimados (U*) do grupo 1
!       U_2.gnu   = grfico de erros estimados (U*) do grupo 2
!       E.gnu     = grfico de erros verdadeiros (E*)
!       E_U.gnu   = grfico de E* e U* do grupo 1
!       E_U_2.gnu = grfico de E* e U* do grupo 2

!    Arquivo texto com informaes bsicas sobre o programa:
!       LEIA-ME_Richardson_3p2.txt

!-------------------------------------------------------------------------------

use resultados

!-------------------------------------------------------------------------------

implicit none

!-------------------------------------------------------------------------------

  call le_dados

  open(out,file=trim(adjustl(caso) ) )

  call data_e_hora ( dia, hora )

  head = trim(title)//" "//trim(dia)//"'"

  call escreve_dados

!-------------------------------------------------------------------------------

  call calcula

  call mostra_resultados

  close (out)

  note_caso = 'notepad ' // trim(adjustl(caso))

  dos = system(note_caso) ! lista arquivo de resultados

!-------------------------------------------------------------------------------

end
