module variaveis

! Objetivo: definir as variveis globais do programa

use portlib
use data_hora

!-------------------------------------------------------------------------------

implicit none

real*16,parameter :: great  = 1.0q+11
real*16,parameter :: limite = 1.0q+10

! *** Variveis principais ***

integer :: variavel ! nmero da varivel de interesse nos arquivos de dados

integer :: nm  ! nmero de malhas

integer :: i   ! ndice das malhas
               ! i = 1  ---> malha mais grossa
               ! i = nm ---> malha mais fina

integer :: m   ! nmero de extrapolaes
               ! m = 1    ---> uma extrapolao
               ! m = nm-1 ---> nmero mximo de extrapolaes

real*16 :: r21 ! razo de h entre as malhas grossa(2) e fina(1)
real*16 :: r32 ! razo de h entre as malhas supergrossa(3) e grossa(2)

real*16 :: pL  ! ordem assinttica do erro verdadeiro
real*16 :: dpL ! variao entre as ordens verdadeiras do erro verdadeiro

integer :: tipo_refino ! tipo de refino de malha (r): 1=constante; 2=varivel

integer :: com_solucao_analitica ! 1 = sim;   0 = no

! dados necessrios apenas para tipo_refino = 2 (r varivel)
! para obter a ordem aparente (pU) atravs do mtodo da bisseo
integer :: numero_iteracoes ! nmero mximo de iteraes
real*16 :: tol              ! tolerncia
real*16 :: pU_min           ! limite inferior de pU
real*16 :: pU_max           ! limite superior de pU

real*16 :: T_exato ! soluo analtica exata

real*16 :: Fs      ! fator de segurana do estimador GCI

! *** Variveis de campo (vetores) com dimenso (1:nm) ***

character*50,dimension(:),allocatable :: arq_numerico ! nomes dos arquivos das solues numricas

real*16,dimension(:),allocatable :: h       ! comprimento de cada elemento da malha

real*16,dimension(:),allocatable :: Th      ! soluo numrica em cada malha

real*16,dimension(:),allocatable     :: pV  ! ordens verdadeiras do erro verdadeiro
character*5,dimension(:),allocatable :: pVt ! pV em formato texto

real*16,dimension(:),allocatable :: Ti_12   ! Th extrapolado com as malhas 1 e 2 e pL
real*16,dimension(:),allocatable :: Ti_13   ! Th extrapolado com as malhas 1 e 3 e pL
real*16,dimension(:),allocatable :: Ti_23   ! Th extrapolado com as malhas 2 e 3 e pL
real*16,dimension(:),allocatable :: Ti_pU   ! Th extrapolado com as malhas 1 a 3 e pU_h
real*16,dimension(:),allocatable :: Ti_bi   ! Th extrapolado com as malhas 1 a 3 e 2 coeficientes
real*16,dimension(:),allocatable :: Ti_tri  ! Th extrapolado com as malhas 1 a 4 e 3 coeficientes
real*16,dimension(:),allocatable :: Tc_12   ! Mdia de Ti_12 com Ti_pU
real*16,dimension(:),allocatable :: Tc_13   ! Mdia de Ti_13 com Ti_pU
real*16,dimension(:),allocatable :: Tbi_pU  ! Th biextrapolado com malhas 1 a 5 e pU_i

real*16,dimension(:),allocatable :: Eh      ! erro verdadeiro de Th

real*16,dimension(:),allocatable :: Ei_12   ! erro verdadeiro de Ti_12
real*16,dimension(:),allocatable :: Ei_13   ! erro verdadeiro de Ti_13
real*16,dimension(:),allocatable :: Ei_23   ! erro verdadeiro de Ti_23
real*16,dimension(:),allocatable :: Ei_pU   ! erro verdadeiro de Ti_pU
real*16,dimension(:),allocatable :: Ei_bi   ! erro verdadeiro de Ti_bi
real*16,dimension(:),allocatable :: Ei_tri  ! erro verdadeiro de Ti_tri
real*16,dimension(:),allocatable :: Ec_12   ! erro verdadeiro de Tc_12
real*16,dimension(:),allocatable :: Ec_13   ! erro verdadeiro de Tc_13
real*16,dimension(:),allocatable :: Ebi_pU  ! erro verdadeiro de Tbi_pU

real*16,dimension(:),allocatable :: pE_h    ! ordem efetiva do erro verdadeiro de Th
real*16,dimension(:),allocatable :: pE_i    ! ordem efetiva do erro verdadeiro de Ti_pU
real*16,dimension(:),allocatable :: pE_c12  ! ordem efetiva do erro verdadeiro de Tc_12
real*16,dimension(:),allocatable :: pE_c13  ! ordem efetiva do erro verdadeiro de Tc_13
real*16,dimension(:),allocatable :: pE_bi   ! ordem efetiva do erro verdadeiro de Tbi_pU

! Estimador delta (Ud)
real*16,dimension(:),allocatable :: Ud      ! erro estimado de Th

! Estimador de Richardson (Uri)
real*16,dimension(:),allocatable :: Uri_12  ! erro estimado de Th com as malhas 1 e 2 e pL
real*16,dimension(:),allocatable :: Uri_13  ! erro estimado de Th com as malhas 1 e 3 e pL
real*16,dimension(:),allocatable :: Uri_23  ! erro estimado de Th com as malhas 2 e 3 e pL
real*16,dimension(:),allocatable :: Uri_pU  ! erro estimado de Th com as malhas 1 a 3 e pU_h
real*16,dimension(:),allocatable :: Uri_p   ! erro estimado de Th com as malhas 1 a 3, pU_h e pL
real*16,dimension(:),allocatable :: Uri_bi  ! erro estimado de Th extrapolado com as malhas 1 a 3 e 2 coeficientes
real*16,dimension(:),allocatable :: Uri_tri ! erro estimado de Th extrapolado com as malhas 1 a 4 e 3 coeficientes

! Estimador GCI de Roache (GCI)
real*16,dimension(:),allocatable :: GCI_pL  ! erro estimado de Th com as malhas 1 e 2 e pL
real*16,dimension(:),allocatable :: GCI_p   ! erro estimado de Th com as malhas 1 a 3, pU_h e pL

! Estimador Convergente (Uc)
real*16,dimension(:),allocatable :: Uc_12   ! erro estimado de Tc_12
real*16,dimension(:),allocatable :: Uc_13   ! erro estimado de Tc_13

real*16,dimension(:),allocatable :: pU_h    ! ordem aparente do erro estimado de Th
real*16,dimension(:),allocatable :: pU_i    ! ordem aparente do erro estimado de Ti_pU
real*16,dimension(:),allocatable :: pU_c12  ! ordem aparente do erro estimado de Tc_12
real*16,dimension(:),allocatable :: pU_c13  ! ordem aparente do erro estimado de Tc_13
real*16,dimension(:),allocatable :: pU_bi   ! ordem aparente do erro estimado de Tbi_pU

integer,dimension(:),allocatable:: it_pU_h   ! nmero de iteraes para obter pU_h
integer,dimension(:),allocatable:: it_pU_i   ! nmero de iteraes para obter pU_i
integer,dimension(:),allocatable:: it_pU_c12 ! nmero de iteraes para obter pU_c12
integer,dimension(:),allocatable:: it_pU_c13 ! nmero de iteraes para obter pU_c13
integer,dimension(:),allocatable:: it_pU_bi  ! nmero de iteraes para obter pU_bi

! *** Variveis de campo (matrizes) com dimenso (1:nm,1:nm-1) ***

real*16,dimension(:,:),allocatable :: Um_12 ! erro estimado de Tm_12 com as malhas 1 e 2 e pV
real*16,dimension(:,:),allocatable :: Tm_12 ! Th multiextrapolado com as malhas 1 e 2 e pV

! *** vriaveis auxiliares ***

integer, parameter :: in   =  7 ! nmero do arquivo de entrada (dados)
integer, parameter :: out  = 10 ! nmero do arquivo de sada principal
integer, parameter :: out2 = 11 ! nmero do arquivo de sada secundrio

character*50  :: caso           ! nome do arquivo de sada
character*90  :: title          ! ttulo do grfico
character*102 :: head           ! ttulo do grfico + dia
character*12  :: dia            ! data da simulao
character*8   :: hora           ! horrio da simulao
character*90  :: note_caso      ! notepad + caso

character*50 :: arq_exato ! nome do arquivo da soluo exata
character*50 :: arq_dados ! nome do arquivo de dados principal

integer :: dos                  ! acessa prompt dos

integer :: j, k ! auxiliares

!-------------------------------------------------------------------------------

end module variaveis